module ZenturioBatteries::Base
  require 'erb'

  # Basic module for generation application parts
  module Generator    

    # Get the list of available options for current generator
    def available_options
      []
    end

    # Execute option for given handler.
    # If there is no such handler - raise Exception
    def execute! handler, hard_mode = false 
      raise Exception.new('No such handler') unless self.methods.include?(handler)

      method = self.method(handler)
      config_list = method.call

      process_config! config_list, hard_mode
    end

    protected

    # Copy necessary files to destination folders.
    # Information about files - in config_list parameter
    #
    # config_list - is a Array and each element is a Hash.
    # each element has 2 keys - :source and :destination
    def process_config! config_list, hard_mode
      config_list.each do |finfo|
        source = finfo[:source]
        dest = finfo[:destination]

        # Setting bindings for templates
        proj_dir = Dir.pwd

        # Rendering templates
        renderer = ERB.new File.new(ZenturioBatteries.base_dir+'/assets'+source).read        
        res = renderer.result binding
        
        # Saving templates to destination
        dest_file = proj_dir+dest

        if File.exists?(dest_file) && !hard_mode
          puts dest_file+' already exists and generation will be skipped.'
        else           
          File.open(dest_file, "w+") do |f|
            f.write(res)
          end
          puts dest_file+' was generated.'
        end

      end
    end

  end
  
end